from Bond_Element import Bond_Element

from ZCRTable import ZCRTable

from datetime import datetime



####################

# Static variables #

#                  #

N = 100            #

DaysInYears = 360  #

#                  #

####################



def NumOfDays(start, end):

    return (end.year - start.year) * 360 + (end.month - start.month) * 30 + (end.day - start.day)



def Maturity(TCoupon):

    global DaysInYears

    return (TCoupon / DaysInYears) + 1



def T_maturity(start, end):

    return NumOfDays(start, end)





def t_coupon(start, end):

    global DaysInYears

    days = NumOfDays(start, end)

    if days < DaysInYears:

        return days

    return days - DaysInYears





def ZeroCouponRate(tCoupon, zcrTable):

    global DaysInYear

    global zero_coupon_rate_table

    r = zero_coupon_rate_table.get(tCoupon)

    if r != None:

        return float(r)

    lastTwoRate = zcrTable.getZCR(tCoupon)

    r = lastTwoRate.values()[1] \

        + ( ((lastTwoRate.values()[1] - lastTwoRate.values()[0]) / (lastTwoRate.keys()[1] - lastTwoRate.keys()[0])) \

        * (tCoupon - lastTwoRate.keys()[1]))

    return float(r)

    

def PVC(couponRate, r_t, tCoupon):

    global DaysInYears

    return float(couponRate / ((1 + r_t ) ** (float(tCoupon) / DaysInYears)))



def PVCTotal(bond_element,TCoupon, new_rates):

    global DaysInYears

    global zero_coupon_rate_table

    global start_time

    list_tCoupon = []

    dict_r_t = {}

    tCoupon = TCoupon

    while(tCoupon > DaysInYears):

        tCoupon -= DaysInYears

        list_tCoupon.append(tCoupon)

    for t in list_tCoupon:

        r_t = ZeroCouponRate(t,zero_coupon_rate_table)

        new_dict = {}

        new_dict[t] = r_t

        new_rates.append(new_dict)

        dict_r_t[t] = PVC(bond_element.Coupon_Rate,r_t,t)

    result = 0

    for pvc in dict_r_t:

        result += dict_r_t[pvc]

    return result



def PVB(couponRate, TCoupon, ytm, M):

    global DaysInYears

    global N

    return float(1 / ((1 + ytm) ** (float(TCoupon) / DaysInYears))) *\

           (N + (couponRate * (((1+ytm)**M) - 1) / ytm))





def ZCP(pvc, pvb):

    return pvb - pvc





def RateMaturityCoupon(couponRate, zcp, TCoupon, new_rates):

    global DaysInYears

    global N

    global zero_coupon_rate_table

    rate_maturity = float((( ((N + couponRate ) / zcp ) ** (float(DaysInYears) / TCoupon)) - 1) )

    new_dict = {}

    new_dict[TCoupon] = rate_maturity

    new_rates.append(new_dict)

    return rate_maturity





def compute_calculation(bond_element):

    global zero_coupon_rate_table

    global start_time

    global DaysInYears

    new_rates = []

    pvc = 0

    print "Bond: " , bond_element ,"\n"

    TCoupon = T_maturity(start_time, bond_element.Maturity)

    print "T Maturity: " , TCoupon;



    M = Maturity(TCoupon)

    print "M: ", M



    if(TCoupon < DaysInYears):

        r_t = bond_element.YTM

        new_dict = {}

        new_dict[TCoupon] = r_t

        new_rates.append(new_dict)

        print "T coupon: " , TCoupon

        for r in new_rates:

            zero_coupon_rate_table.addElement(r)

        #print zero_coupon_rate_table

        return



    elif( M <= 2 ):



        tCoupon = t_coupon(start_time, bond_element.Maturity)

        print "T coupon: " , tCoupon



        r_t = ZeroCouponRate(tCoupon, zero_coupon_rate_table)

        new_dict = {}

        new_dict[TCoupon] = r_t

        new_rates.append(new_dict)

        print "Rate of the coupon: ", r_t



        pvc = PVC(bond_element.Coupon_Rate, r_t, tCoupon)

        print "PV(coupon): ", pvc



    else:

        pvc = PVCTotal(bond_element,TCoupon, new_rates)

        print "PV(coupon): ", pvc



    pvb = PVB(bond_element.Coupon_Rate, TCoupon, bond_element.YTM,M)

    print "PV(Bond): ", float(pvb)



    zcp = ZCP(pvc, pvb)

    print "ZCP: ", zcp



    r_Maturity = RateMaturityCoupon(bond_element.Coupon_Rate, zcp, TCoupon, new_rates)

    print "Maturity Rate: " , r_Maturity

    for r in new_rates:

        zero_coupon_rate_table.addElement(r)

    #print zero_coupon_rate_table



#####################################################################################

#                    Definition of the Zero Coupon Rate Table                       #

zero_coupon_rate_table = ZCRTable({30: 0.0002, 60: 0.0002, 90: 0.0001, 120: 0.0002})#

#                                                                                   #

#####################################################################################



#####################################################################################

#                                   Start Time                                      #

start_time = datetime.strptime("21112014", "%d%m%Y").date()                         #

#                                                                                   #

#####################################################################################





#####################################################################################

#                                    Bond Elements                                  #

bond_element1 = Bond_Element("SO-1049",0.03,"12082015",4.5)                       #

bond_element2 = Bond_Element("SO-1050(2Y)", -0.01, "12072016",3)             #

bond_element3 = Bond_Element("SO-1051",0.04 ,"12082017",3.75)                  #

bond_element4 = Bond_Element("SO-1052(5Y)", 0.24,"12032019",4.25)              #

bond_element5 = Bond_Element("SO-1047", 0.56,"01122020",5.00)                  #

bond_element6 = Bond_Element("SO-1054", 0.78,"01062022",3.50)                  #

bond_element7 = Bond_Element("SO-1057", 0.98,"13112023",1.50)                  #

bond_element8 = Bond_Element("SO-1058(10Y)", 1.11,"12052025",2.50)             #

bond_element9 = Bond_Element("SO-1056", 1.61,"01062032",2.25)                  #

bond_element10 = Bond_Element("SO-1053", 1.78,"30032039",3.50)                 #

#                                                                                   #

#####################################################################################





#####################################################################################

#                                Compute Calculations                               #

compute_calculation(bond_element1)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element2)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element3)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element4)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element5)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element6)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element7)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element8)                                                  #

#print "\n" , zero_coupon_rate_table                                                #

compute_calculation(bond_element9)                                                  #

#print "\n" , zero_coupon_rate_table                                                #                                

compute_calculation(bond_element10)                                                 #

print "\n" , zero_coupon_rate_table                                                 #

#                                                                                   #

#####################################################################################



import matplotlib.pyplot as plt

import numpy as np



plt.plot( zero_coupon_rate_table.dict.keys(), zero_coupon_rate_table.dict.values(), "ro")

plt.title("Zero-coupon yield curve")

plt.xlabel("Days")

plt.ylabel("Zero Coupon Rates")

plt.show() 



discounts_factor = []



for t in zero_coupon_rate_table.dict:

    discounts_factor.append(np.exp(-zero_coupon_rate_table.dict[t]*t/360))

    print np.exp(-zero_coupon_rate_table.dict[t]*t/360)

plt.plot(zero_coupon_rate_table.dict.keys(), discounts_factor)

plt.title("Discount function")

plt.xlabel("Days")

plt.ylabel("discont_factor")

plt.show() 